@@ -98,11 +98,11 @@ static int verify_convert_cmd(struct fio_net_cmd *cmd)
 	cmd->version	= le16_to_cpu(cmd->version);
 	cmd->opcode	= le16_to_cpu(cmd->opcode);
 	cmd->flags	= le32_to_cpu(cmd->flags);
-	cmd->serial	= le64_to_cpu(cmd->serial);
+	cmd->tag	= le64_to_cpu(cmd->tag);
 	cmd->pdu_len	= le32_to_cpu(cmd->pdu_len);
 
 	switch (cmd->version) {
-	case FIO_SERVER_VER3:
+	case FIO_SERVER_VER:
 		break;
 	default:
 		log_err("fio: bad server cmd version %d\n", cmd->version);
@@ -205,10 +205,11 @@ void fio_net_cmd_crc(struct fio_net_cmd *cmd)
 		cmd->pdu_crc16 = __cpu_to_le16(crc16(cmd->payload, pdu_len));
 }
 
-int fio_net_send_cmd(int fd, uint16_t opcode, const void *buf, off_t size)
+int fio_net_send_cmd(int fd, uint16_t opcode, const void *buf, off_t size,
+		     uint64_t tag)
 {
-	struct fio_net_cmd *cmd;
-	size_t this_len;
+	struct fio_net_cmd *cmd = NULL;
+	size_t this_len, cur_len = 0;
 	int ret;
 
 	do {
@@ -216,9 +217,15 @@ int fio_net_send_cmd(int fd, uint16_t opcode, const void *buf, off_t size)
 		if (this_len > FIO_SERVER_MAX_PDU)
 			this_len = FIO_SERVER_MAX_PDU;
 
-		cmd = malloc(sizeof(*cmd) + this_len);
+		if (!cmd || cur_len < sizeof(*cmd) + this_len) {
+			if (cmd)
+				free(cmd);
 
-		fio_init_net_cmd(cmd, opcode, buf, this_len);
+			cur_len = sizeof(*cmd) + this_len;
+			cmd = malloc(cur_len);
+		}
+
+		fio_init_net_cmd(cmd, opcode, buf, this_len, tag);
 
 		if (this_len < size)
 			cmd->flags = __cpu_to_le32(FIO_NET_CMD_F_MORE);
@@ -226,19 +233,21 @@ int fio_net_send_cmd(int fd, uint16_t opcode, const void *buf, off_t size)
 		fio_net_cmd_crc(cmd);
 
 		ret = fio_send_data(fd, cmd, sizeof(*cmd) + this_len);
-		free(cmd);
 		size -= this_len;
 		buf += this_len;
 	} while (!ret && size);
 
+	if (cmd)
+		free(cmd);
+
 	return ret;
 }
 
-int fio_net_send_simple_cmd(int sk, uint16_t opcode, uint64_t serial)
+int fio_net_send_simple_cmd(int sk, uint16_t opcode, uint64_t tag)
 {
 	struct fio_net_cmd cmd;
 
-	fio_init_net_cmd(&cmd, opcode, NULL, 0);
+	fio_init_net_cmd(&cmd, opcode, NULL, 0, tag);
 	fio_net_cmd_crc(&cmd);
 
 	return fio_send_data(sk, &cmd, sizeof(cmd));
@@ -270,24 +279,36 @@ static int handle_job_cmd(struct fio_net_cmd *cmd)
 
 static int handle_jobline_cmd(struct fio_net_cmd *cmd)
 {
-	struct cmd_line_pdu *pdu = (struct cmd_line_pdu *) cmd->payload;
-	char *argv[FIO_NET_CMD_JOBLINE_ARGV];
+	void *pdu = cmd->payload;
+	struct cmd_single_line_pdu *cslp;
+	struct cmd_line_pdu *clp;
+	unsigned long offset;
+	char **argv;
 	int ret, i;
 
-	pdu->argc = le16_to_cpu(pdu->argc);
+	clp = pdu;
+	clp->lines = le16_to_cpu(clp->lines);
+	argv = malloc(clp->lines * sizeof(char *));
+	offset = sizeof(*clp);
 
-	dprint(FD_NET, "server: %d command line args\n", pdu->argc);
+	dprint(FD_NET, "server: %d command line args\n", clp->lines);
 
-	for (i = 0; i < pdu->argc; i++) {
-		argv[i] = (char *) pdu->argv[i];
+	for (i = 0; i < clp->lines; i++) {
+		cslp = pdu + offset;
+		argv[i] = (char *) cslp->text;
+
+		offset += sizeof(*cslp) + le16_to_cpu(cslp->len);
 		dprint(FD_NET, "server: %d: %s\n", i, argv[i]);
 	}
 
-	if (parse_cmd_line(pdu->argc, argv)) {
+	if (parse_cmd_line(clp->lines, argv)) {
 		fio_server_send_quit_cmd();
+		free(argv);
 		return -1;
 	}
 
+	free(argv);
+
 	fio_net_send_simple_cmd(server_fd, FIO_NET_CMD_START, 0);
 
 	ret = exec_run();
@@ -312,14 +333,53 @@ static int handle_probe_cmd(struct fio_net_cmd *cmd)
 	probe.os	= FIO_OS;
 	probe.arch	= FIO_ARCH;
 
-	return fio_net_send_cmd(server_fd, FIO_NET_CMD_PROBE, &probe, sizeof(probe));
+	return fio_net_send_cmd(server_fd, FIO_NET_CMD_PROBE, &probe, sizeof(probe), 0);
+}
+
+static int handle_send_eta_cmd(struct fio_net_cmd *cmd)
+{
+	struct jobs_eta *je;
+	size_t size;
+	int i;
+
+	size = sizeof(*je) + thread_number * sizeof(char);
+	je = malloc(size);
+	memset(je, 0, size);
+
+	if (!calc_thread_status(je, 1)) {
+		free(je);
+		return 0;
+	}
+
+	dprint(FD_NET, "server sending status\n");
+
+	je->nr_running		= cpu_to_le32(je->nr_running);
+	je->nr_ramp		= cpu_to_le32(je->nr_ramp);
+	je->nr_pending		= cpu_to_le32(je->nr_pending);
+	je->files_open		= cpu_to_le32(je->files_open);
+	je->m_rate		= cpu_to_le32(je->m_rate);
+	je->t_rate		= cpu_to_le32(je->t_rate);
+	je->m_iops		= cpu_to_le32(je->m_iops);
+	je->t_iops		= cpu_to_le32(je->t_iops);
+
+	for (i = 0; i < 2; i++) {
+		je->rate[i]	= cpu_to_le32(je->rate[i]);
+		je->iops[i]	= cpu_to_le32(je->iops[i]);
+	}
+
+	je->elapsed_sec		= cpu_to_le32(je->nr_running);
+	je->eta_sec		= cpu_to_le64(je->eta_sec);
+
+	fio_net_send_cmd(server_fd, FIO_NET_CMD_ETA, je, size, cmd->tag);
+	free(je);
+	return 0;
 }
 
 static int handle_command(struct fio_net_cmd *cmd)
 {
 	int ret;
 
-	dprint(FD_NET, "server: got opcode %d\n", cmd->opcode);
+	dprint(FD_NET, "server: got opcode %d, pdu=%u\n", cmd->opcode, cmd->pdu_len);
 
 	switch (cmd->opcode) {
 	case FIO_NET_CMD_QUIT:
@@ -337,6 +397,9 @@ static int handle_command(struct fio_net_cmd *cmd)
 	case FIO_NET_CMD_PROBE:
 		ret = handle_probe_cmd(cmd);
 		break;
+	case FIO_NET_CMD_SEND_ETA:
+		ret = handle_send_eta_cmd(cmd);
+		break;
 	default:
 		log_err("fio: unknown opcode: %d\n", cmd->opcode);
 		ret = 1;
@@ -462,12 +525,12 @@ out:
 	return exitval;
 }
 
-int fio_server_text_output(const char *buf, unsigned int len)
+int fio_server_text_output(const char *buf, size_t len)
 {
 	if (server_fd != -1)
-		return fio_net_send_cmd(server_fd, FIO_NET_CMD_TEXT, buf, len);
+		return fio_net_send_cmd(server_fd, FIO_NET_CMD_TEXT, buf, len, 0);
 
-	return fwrite(buf, len, 1, f_err);
+	return log_local_buf(buf, len);
 }
 
 static void convert_io_stat(struct io_stat *dst, struct io_stat *src)
@@ -578,7 +641,7 @@ void fio_server_send_ts(struct thread_stat *ts, struct group_run_stats *rs)
 
 	convert_gs(&p.rs, rs);
 
-	fio_net_send_cmd(server_fd, FIO_NET_CMD_TS, &p, sizeof(p));
+	fio_net_send_cmd(server_fd, FIO_NET_CMD_TS, &p, sizeof(p), 0);
 }
 
 void fio_server_send_gs(struct group_run_stats *rs)
@@ -588,47 +651,7 @@ void fio_server_send_gs(struct group_run_stats *rs)
 	dprint(FD_NET, "server sending group run stats\n");
 
 	convert_gs(&gs, rs);
-	fio_net_send_cmd(server_fd, FIO_NET_CMD_GS, &gs, sizeof(gs));
-}
-
-void fio_server_send_status(void)
-{
-	struct jobs_eta *je;
-	size_t size;
-	void *buf;
-	int i;
-
-	size = sizeof(*je) + thread_number * sizeof(char);
-	buf = malloc(size);
-	memset(buf, 0, size);
-	je = buf;
-
-	if (!calc_thread_status(je)) {
-		free(je);
-		return;
-	}
-
-	dprint(FD_NET, "server sending status\n");
-
-	je->nr_running		= cpu_to_le32(je->nr_running);
-	je->nr_ramp		= cpu_to_le32(je->nr_ramp);
-	je->nr_pending		= cpu_to_le32(je->nr_pending);
-	je->files_open		= cpu_to_le32(je->files_open);
-	je->m_rate		= cpu_to_le32(je->m_rate);
-	je->t_rate		= cpu_to_le32(je->t_rate);
-	je->m_iops		= cpu_to_le32(je->m_iops);
-	je->t_iops		= cpu_to_le32(je->t_iops);
-
-	for (i = 0; i < 2; i++) {
-		je->rate[i]	= cpu_to_le32(je->rate[i]);
-		je->iops[i]	= cpu_to_le32(je->iops[i]);
-	}
-
-	je->elapsed_sec		= cpu_to_le32(je->nr_running);
-	je->eta_sec		= cpu_to_le64(je->eta_sec);
-
-	fio_net_send_cmd(server_fd, FIO_NET_CMD_ETA, buf, size);
-	free(je);
+	fio_net_send_cmd(server_fd, FIO_NET_CMD_GS, &gs, sizeof(gs), 0);
 }
 
 int fio_server_log(const char *format, ...)
@@ -901,31 +924,93 @@ static void server_signal_handler(void)
 	sigaction(SIGTERM, &act, NULL);
 }
 
-int fio_start_server(int daemonize)
+static int check_existing_pidfile(const char *pidfile)
+{
+	struct stat sb;
+	char buf[16];
+	pid_t pid;
+	FILE *f;
+
+	if (stat(pidfile, &sb))
+		return 0;
+
+	f = fopen(pidfile, "r");
+	if (!f)
+		return 0;
+
+	if (fread(buf, sb.st_size, 1, f) <= 0) {
+		fclose(f);
+		return 1;
+	}
+	fclose(f);
+
+	pid = atoi(buf);
+	if (kill(pid, SIGCONT) < 0)
+		return 0;
+
+	return 1;
+}
+
+static int write_pid(pid_t pid, const char *pidfile)
+{
+	FILE *fpid;
+
+	fpid = fopen(pidfile, "w");
+	if (!fpid) {
+		log_err("fio: failed opening pid file %s\n", pidfile);
+		return 1;
+	}
+
+	fprintf(fpid, "%u\n", (unsigned int) pid);
+	fclose(fpid);
+	return 0;
+}
+
+/*
+ * If pidfile is specified, background us.
+ */
+int fio_start_server(char *pidfile)
 {
 	pid_t pid;
+	int ret;
 
 	server_signal_handler();
 
-	if (!daemonize)
+	if (!pidfile)
 		return fio_server();
 
-	openlog("fio", LOG_NDELAY|LOG_NOWAIT|LOG_PID, LOG_USER);
+	if (check_existing_pidfile(pidfile)) {
+		log_err("fio: pidfile %s exists and server appears alive\n",
+								pidfile);
+		return -1;
+	}
+
 	pid = fork();
 	if (pid < 0) {
-		syslog(LOG_ERR, "failed server fork");
+		log_err("fio: failed server fork: %s", strerror(errno));
+		free(pidfile);
 		return -1;
-	} else if (pid)
-		exit(0);
+	} else if (pid) {
+		int ret = write_pid(pid, pidfile);
+
+		exit(ret);
+	}
 
 	setsid();
+	openlog("fio", LOG_NDELAY|LOG_NOWAIT|LOG_PID, LOG_USER);
+	log_syslog = 1;
 	close(STDIN_FILENO);
 	close(STDOUT_FILENO);
 	close(STDERR_FILENO);
 	f_out = NULL;
 	f_err = NULL;
-	log_syslog = 1;
-	return fio_server();
+
+	ret = fio_server();
+
+	closelog();
+	unlink(pidfile);
+	free(pidfile);
+	return ret;
 }
 
 void fio_server_set_arg(const char *arg)
